@@ -16,7 +16,7 @@ import os
 import uuid
 from copy import deepcopy
 from pathlib import Path
-from typing import Dict, List, Optional, Tuple, Union, Any
+from typing import Dict, List, Optional, Tuple, Union, Any, Iterable, Callable
 
 import dill
 import traceback
@@ -28,12 +28,12 @@ from tqdm.auto import tqdm
 from .feature import FeatureDescriptor, MultipleFeatureDescriptors
 from .logger import logger
 from .segmenter import StridedRollingFactory, StridedRolling
-from .utils import _determine_bounds
+from .utils import _determine_bounds, _check_start_end_array
 from ..features.function_wrapper import FuncWrapper
 from ..utils.attribute_parsing import AttributeParser
-from ..utils.data import to_list, to_series_list, flatten
+from ..utils.data import to_list, to_tuple, to_series_list, flatten
 from ..utils.logging import delete_logging_handlers, add_logging_handler
-from ..utils.time import timedelta_to_str
+from ..utils.time import parse_time_arg, timedelta_to_str
 
 
 class FeatureCollection:
@@ -69,18 +69,19 @@ class FeatureCollection:
                 FeatureDescriptor,
                 MultipleFeatureDescriptors,
                 FeatureCollection,
-                List[Union[FeatureDescriptor, MultipleFeatureDescriptors, FeatureCollection]],
+                List[
+                    Union[
+                        FeatureDescriptor, MultipleFeatureDescriptors, FeatureCollection
+                    ]
+                ],
             ]
         ] = None,
     ):
         # The feature collection is a dict with keys of type:
-        #   tuple(tuple(str), float OR pd.timedelta, float OR pd.timedelta)
-        # The outer tuple's values correspond to (series_key(s), window, stride)
+        #   tuple(tuple(str), float OR pd.timedelta)
+        # The outer tuple's values correspond to (series_key(s), window)
         self._feature_desc_dict: Dict[
-            Tuple[
-                Tuple[str, ...], Union[float, pd.Timedelta], Union[float, pd.Timedelta]
-            ],
-            List[FeatureDescriptor],
+            Tuple[Tuple[str, ...], Union[float, pd.Timedelta]], List[FeatureDescriptor]
         ] = {}
 
         if feature_descriptors:
@@ -103,23 +104,79 @@ class FeatureCollection:
             set(flatten([fr_key[0] for fr_key in self._feature_desc_dict.keys()]))
         )
 
+    def get_nb_output_features(self) -> int:
+        """Return the number of output features in this feature collection.
+
+        Returns
+        -------
+        int
+            The number of output features in this feature collection.
+
+        """
+        fd_list: Iterable[FeatureDescriptor] = flatten(self._feature_desc_dict.values())
+        return sum(fd.get_nb_output_features() for fd in fd_list)
+
+    def _get_nb_output_features_without_window(self) -> int:
+        """Return the number of output features in this feature collection, without
+        using the window as a unique identifier.
+
+        This is relevant for when the window value(s) are overridden by passing
+        `segment_start_idxs` and `segment_end_idxs` to the `calculate` method.
+
+        Returns
+        -------
+        int:
+            The number of output features in this feature collection without using the
+            window as a unique identifier.
+
+        """
+        return len(
+            set(
+                (series, o)
+                for (series, _), fd_list in self._feature_desc_dict.items()
+                for fd in fd_list
+                for o in fd.function.output_names
+            )
+        )
+
     @staticmethod
     def _get_collection_key(
         feature: FeatureDescriptor,
-    ) -> Tuple[tuple, Union[pd.Timedelta, float], Union[pd.Timedelta, float]]:
-        # Note: `window` & `stride` properties can either be a pd.Timedelta or an int
-        return feature.series_name, feature.window, feature.stride
+    ) -> Tuple[Tuple[str, ...], Union[pd.Timedelta, float, None]]:
+        # Note: `window` property can be either a pd.Timedelta or a float or None
+        # assert feature.window is not None
+        return feature.series_name, feature.window
 
-    def _check_feature_descriptors(self):
+    def _check_feature_descriptors(
+        self,
+        skip_none: bool,
+        calc_stride: Optional[Union[float, pd.Timedelta, None]] = None,
+    ):
         """Verify whether all added FeatureDescriptors imply the same-input data type.
 
         If this condition is not met, a warning will be raised.
 
+        Parameters
+        ----------
+        skip_none: bool
+            Whether to include None stride values in the checks.
+        calc_stride: Union[float, pd.Timedelta, None], optional
+            The `FeatureCollection.calculate` its stride argument, by default None.
+            This stride takes precedence over a `FeatureDescriptor` its stride when
+            it is not None.
+
         """
-        dtype_set = set(
-            AttributeParser.determine_type([win, stride])
-            for _, win, stride in self._feature_desc_dict.keys()
-        )
+        dtype_set = set()
+        for series_names, win in self._feature_desc_dict.keys():
+            for fd in self._feature_desc_dict[(series_names, win)]:
+                stride = calc_stride if calc_stride is not None else fd.stride
+                if skip_none and stride is None:
+                    dtype_set.add(AttributeParser.determine_type(win))
+                else:
+                    dtype_set.add(
+                        AttributeParser.determine_type([win] + to_list(stride))
+                    )
+
         if len(dtype_set) > 1:
             warnings.warn(
                 "There are multiple FeatureDescriptor window-stride "
@@ -200,7 +257,7 @@ class FeatureCollection:
                 raise TypeError(f"type: {type(feature)} is not supported - {feature}")
 
         # After adding the features, check whether the descriptors are compatible
-        self._check_feature_descriptors()
+        self._check_feature_descriptors(skip_none=True)
 
     @staticmethod
     def _executor(idx: int):
@@ -208,14 +265,21 @@ class FeatureCollection:
         stroll, function = get_stroll_func(idx)
         return stroll.apply_func(function)
 
+    # def _get_stroll(self, kwargs):
+    #     return StridedRollingFactory.get_segmenter(**kwargs)
+
     def _stroll_feat_generator(
         self,
         series_dict: Dict[str, pd.Series],
+        calc_stride: Union[List[Union[float, pd.Timedelta]], None],
+        segment_start_idxs: Union[np.ndarray, None],
+        segment_end_idxs: Union[np.ndarray, None],
         start_idx: Any,
         end_idx: Any,
         window_idx: str,
+        include_final_window: bool,
         approve_sparsity: bool,
-    ) -> List[Tuple[StridedRolling, FuncWrapper]]:
+    ) -> Callable[[int], Tuple[StridedRolling, FuncWrapper]]:
         # --- Future work ---
         # We could also make the StridedRolling creation multithreaded
         # Very low priority because the STROLL __init__ is rather efficient!
@@ -224,21 +288,26 @@ class FeatureCollection:
             [len(self._feature_desc_dict[k]) for k in keys_wins_strides]
         )
 
-        def get_stroll_function(idx):
+        def get_stroll_function(idx) -> Tuple[StridedRolling, FuncWrapper]:
             key_idx = np.searchsorted(lengths, idx, "right")  # right bc idx starts at 0
-            key, win, stride = keys_wins_strides[key_idx]
+            key, win = keys_wins_strides[key_idx]
+
             feature = self._feature_desc_dict[keys_wins_strides[key_idx]][
                 idx - lengths[key_idx]
             ]
+            stride = feature.stride if calc_stride is None else calc_stride
             function: FuncWrapper = feature.function
             # The factory method will instantiate the right StridedRolling object
             stroll_arg_dict = dict(
                 data=[series_dict[k] for k in key],
                 window=win,
-                stride=stride,
+                strides=stride,
+                segment_start_idxs=segment_start_idxs,
+                segment_end_idxs=segment_end_idxs,
                 start_idx=start_idx,
                 end_idx=end_idx,
                 window_idx=window_idx,
+                include_final_window=include_final_window,
                 approve_sparsity=approve_sparsity,
                 func_data_type=function.input_type,
             )
@@ -252,11 +321,37 @@ class FeatureCollection:
             len(self._feature_desc_dict[k]) for k in self._feature_desc_dict.keys()
         )
 
+    def _check_no_multiple_windows(self):
+        assert (
+            self._get_nb_output_features_without_window()
+            == self.get_nb_output_features()
+        ), (
+            "When using `segment_XXX_idxs`; each output name - series_input combination"
+            + " can only have 1 window (or None)"
+        )
+
+    @staticmethod
+    def _process_segment_idxs(
+        segment_idxs: Union[list, np.ndarray, pd.Series, pd.Index]
+    ) -> np.ndarray:
+        if hasattr(segment_idxs, "values"):
+            segment_idxs = segment_idxs.values
+        segment_idxs = np.asarray(segment_idxs)
+        if segment_idxs.ndim > 1:
+            segment_idxs = segment_idxs.squeeze()  # remove singleton dimensions
+        return segment_idxs
+
     def calculate(
         self,
         data: Union[pd.Series, pd.DataFrame, List[Union[pd.Series, pd.DataFrame]]],
+        stride: Optional[Union[float, str, pd.Timedelta, List, None]] = None,
+        segment_start_idxs: Optional[
+            Union[list, np.ndarray, pd.Series, pd.Index]
+        ] = None,
+        segment_end_idxs: Optional[Union[list, np.ndarray, pd.Series, pd.Index]] = None,
         return_df: Optional[bool] = False,
         window_idx: Optional[str] = "end",
+        include_final_window: Optional[bool] = False,
         bound_method: Optional[str] = "inner",
         approve_sparsity: Optional[bool] = False,
         show_progress: Optional[bool] = False,
@@ -265,15 +360,6 @@ class FeatureCollection:
     ) -> Union[List[pd.DataFrame], pd.DataFrame]:
         """Calculate features on the passed data.
 
-        Notes
-        ------
-        * The (column-)names of the series in `data` represent the `series_names`.
-        * If a `logging_file_path` is provided, the execution (time) info can be
-          retrieved by calling `logger.get_feature_logs(logging_file_path)`.
-          Be aware that the `logging_file_path` gets cleared before the logger pushes
-          logged messages. Hence, one should use a separate logging file for each
-          constructed processing and feature instance with this library.
-
         Parameters
         ----------
         data : Union[pd.Series, pd.DataFrame, List[Union[pd.Series, pd.DataFrame]]]
@@ -283,8 +369,56 @@ class FeatureCollection:
             * each Series / DataFrame must have a sortable index. This index represents
             the sequence position of the corresponding values, the index can be either
             numeric or a ``pd.DatetimeIndex``.
-            * each Series / DataFrame index must be comparable.with all others
+            * each Series / DataFrame index must be comparable with all others
             * we assume that each series-name / dataframe-column-name is unique.
+        stride: Union[float, str, pd.Timedelta, List[Union[float, str, pd.Timedelta], None], optional
+            The stride size. By default None. This argument supports multiple types: \n
+            * If None, the stride of the `FeatureDescriptor` objects will be used.
+            * If the type is an `float` or an `int`, its value represents the series:\n
+                - its stride **range** when a **non time-indexed** series is passed.
+                - the stride in **number of samples**, when a **time-indexed** series
+                is passed (must then be and `int`)
+            * If the stride's type is a `pd.Timedelta`, the stride size represents
+            the stride-time delta. The passed data **must have a time-index**.
+            * If a `str`, it must represent a stride-time-delta-string. Hence, the
+            **passed data must have a time-index**. \n
+            .. Note::
+                When set, this stride argument takes precedence over the stride property
+                of the `FeatureDescriptor`s in this `FeatureCollection` (i.e., when a
+                not None value for `stride` passed to this method).
+        segment_start_idxs: Union[list, np.ndarray, pd.Series, pd.Index], optional
+            The start indices of the segments. If None, the start indices will be
+            computed from the data using either:\n
+            - the `segment_end_idxs` - the `window` size property of the
+                `FeatureDescriptor` in this `FeatureCollection` (if `segment_end_idxs`
+                is not None)
+            - strided-window rolling on the data using `window` and `stride` of the
+                `FeatureDescriptor` in this `FeatureCollection` (if `segment_end_idxs`
+                 is also None). (Note that the `stride` argument of this method takes
+                 precedence over the `stride` property of the `FeatureDescriptor`s).
+            By default None.
+        segment_end_idxs: Union[list, np.ndarray, pd.Series, pd.Index], optional
+            The end indices for the segmented windows. If None, the end indices will be
+            computed from the data using either:\n
+            - the `segment_start_idxs` + the `window` size property of the
+                `FeatureDescriptor` in this `FeatureCollection` (if `segment_start_idxs`
+                is not None)
+            - strided-window rolling on the data using `window` and `stride` of the
+                `FeatureDescriptor` in this `FeatureCollection` (if `segment_start_idxs`
+                 is also None). (Note that the `stride` argument of this method takes
+                 precedence over the `stride` property of the `FeatureDescriptor`s).
+            By default None.
+
+            ..Note::
+                When passing both `segment_start_idxs` and `segment_end_idxs`, these two
+                arguments must have the same length and every start index must be <=
+                than the corresponding end index.
+                Note that passing both arguments, discards any meaning of the `window`
+                and `stride` values (as these segment indices define the segmented data,
+                and thus no strided-window rolling index calculation has to be executed).
+                As such, the user can create variable-length segmented windows. However,
+                in such cases, the user should be weary that the feature functions are
+                invariant to these (potentially variable-length) windows.
         return_df : bool, optional
             Whether the output needs to be a DataFrame or a list thereof, by default
             False. If `True` the output dataframes will be merged to a DataFrame with an
@@ -292,8 +426,38 @@ class FeatureCollection:
         window_idx : str, optional
             The window's index position which will be used as index for the
             feature_window aggregation. Must be either of: `["begin", "middle", "end"]`.
-            by default "end". All features in this collection will use the same
+            by **default "end"**. All features in this collection will use the same
             window_idx.
+
+            ..Note::
+                `window_idx`="end" uses the window's end (= right open bound) as
+                output index. \n
+                `window_idx`="begin" uses the window's start idx (= left closed bound)
+                as output index.
+        include_final_window : bool, optional
+            Whether the final (possibly incomplete) window should be included in the
+            strided-window segmentation, by default False.
+
+            .. Note::
+                The remarks below apply when `include_final_window` is set to True.
+                The user should be aware that the last window *might* be incomplete,
+                i.e.;
+
+                - when equally sampled, the last window *might* be smaller than the
+                  the other windows.
+                - when not equally sampled, the last window *might* not include all the
+                  data points (as the begin-time + window-size comes after the last data
+                  point).
+
+                Note, that when equally sampled, the last window *will* be a full window
+                when:
+
+                - the stride is the sampling rate of the data (or stride = 1 for
+                sample-based configurations).<br>
+                **Remark**: that when `include_final_window` is set to False, the last
+                window (which is a full) window will not be included!
+                - *(len * sampling_rate - window_size) % stride = 0*. Remark that the
+                  above case is a base case of this.
         bound_method: str, optional
             The start-end bound methodology which is used to generate the slice ranges
             when ``data`` consists of multiple series / columns.
@@ -320,18 +484,17 @@ class FeatureCollection:
             If n_jobs is either 0 or 1, the code will be executed sequentially without
             creating a process pool. This is very useful when debugging, as the stack
             trace will be more comprehensible.
-            .. note:
+            .. note::
                 Multiprocessed execution is not supported on Windows. Even when,
                 `n_jobs` is set > 1, the feature extraction will still be executed
                 sequentially.
                 Why do we not support multiprocessing on Windows; see this issue
                 https://github.com/predict-idlab/tsflex/issues/51
 
-
             .. tip::
                 It takes on avg. _300ms_ to schedule everything with
                 multiprocessing. So if your sequential feature extraction code runs
-                faster than ~1.5s, it might not be worth it to parallelize the process
+                faster than ~1s, it might not be worth it to parallelize the process
                 (and thus better leave `n_jobs` to 0 or 1).
 
         Returns
@@ -344,6 +507,16 @@ class FeatureCollection:
         KeyError
             Raised when a required key is not found in `data`.
 
+        Notes
+        ------
+        * The (column-)names of the series in `data` represent the `series_names`.
+        * If a `logging_file_path` is provided, the execution (time) info can be
+          retrieved by calling `logger.get_feature_logs(logging_file_path)`.
+          Be aware that the `logging_file_path` gets cleared before the logger pushes
+          logged messages. Hence, one should use a separate logging file for each
+          constructed processing and feature instance with this library.
+
+
         """
         # Delete other logging handlers
         delete_logging_handlers(logger)
@@ -351,11 +524,61 @@ class FeatureCollection:
         if logging_file_path:
             f_handler = add_logging_handler(logger, logging_file_path)
 
+        # Convert to numpy array (if necessary)
+        if segment_start_idxs is not None:
+            segment_start_idxs = FeatureCollection._process_segment_idxs(segment_start_idxs)
+        if segment_end_idxs is not None:
+            segment_end_idxs = FeatureCollection._process_segment_idxs(segment_end_idxs)
+
+        if segment_start_idxs is not None and segment_end_idxs is not None:
+            # Check if segment indices have same length and whether every start idx
+            # <= end idx
+            _check_start_end_array(segment_start_idxs, segment_end_idxs)
+            # Check if there is either 1 or No(ne) window value for every output name -
+            # input_series combination
+            self._check_no_multiple_windows()
+
+        if segment_start_idxs is None or segment_end_idxs is None:
+            assert all(
+                fd.window is not None
+                for fd in flatten(self._feature_desc_dict.values())
+            ), (
+                "Each feature descriptor must have a window when not both "
+                + "segment_start_idxs and segment_end_idxs are provided"
+            )
+
+        if stride is None and segment_start_idxs is None and segment_end_idxs is None:
+            assert all(
+                fd.stride is not None
+                for fd in flatten(self._feature_desc_dict.values())
+            ), (
+                "Each feature descriptor must have a stride when no stride or "
+                + "segment indices are passed to this method!"
+            )
+        elif stride is not None and (
+            segment_start_idxs is not None or segment_end_idxs is not None
+        ):
+            raise ValueError(
+                "The stride and any segment index argument cannot be set together! "
+                + "At least one of both should be None."
+            )
+
+        if stride is not None:
+            # Verify whether the stride complies with the input data dtype
+            stride = [
+                parse_time_arg(s) if isinstance(s, str) else s for s in to_list(stride)
+            ]
+            self._check_feature_descriptors(skip_none=False, calc_stride=stride)
+
         # Convert the data to a series_dict
         series_dict: Dict[str, pd.Series] = {}
         for s in to_series_list(data):
             if not s.index.is_monotonic_increasing:
-                # TODO -> maybe raise a warning?
+                warnings.warn(
+                    f"The index of series '{s.name}' is not monotonic increasing. "
+                    + "The series will be sorted by the index.",
+                    RuntimeWarning,
+                )
                 s = s.sort_index(ascending=True, inplace=False, ignore_index=False)
 
             # Assert the assumptions we make!
@@ -364,22 +587,35 @@ class FeatureCollection:
             if s.name in self.get_required_series():
                 series_dict[str(s.name)] = s
 
-        # determing the bounds of the series dict items and slice on them
+        # Determine the bounds of the series dict items and slice on them
+        # TODO: is dit wel nodig `hier? want we doen dat ook in de strided rolling
         start, end = _determine_bounds(bound_method, list(series_dict.values()))
         series_dict = {
-            n: s.loc[s.index.dtype.type(start) : s.index.dtype.type(end)]  # TODO: check memory efficiency of ths
+            n: s.loc[
+                s.index.dtype.type(start) : s.index.dtype.type(end)
+            ]  # TODO: check memory efficiency of ths
             for n, s, in series_dict.items()
         }
 
         # Note: this variable has a global scope so this is shared in multiprocessing
-        # TODO: try to make this more efficient
+        # TODO: try to make this more efficient (but is not really the bottleneck)
         global get_stroll_func
         get_stroll_func = self._stroll_feat_generator(
-            series_dict, start, end, window_idx, approve_sparsity
+            series_dict,
+            calc_stride=stride,
+            segment_start_idxs=segment_start_idxs,
+            segment_end_idxs=segment_end_idxs,
+            start_idx=start,
+            end_idx=end,
+            window_idx=window_idx,
+            include_final_window=include_final_window,
+            approve_sparsity=approve_sparsity,
         )
         nb_stroll_funcs = self._get_stroll_feat_length()
 
-        if os.name == "nt":  # On Windows no multiprocessing is supported, see https://github.com/predict-idlab/tsflex/issues/51
+        if (
+            os.name == "nt"
+        ):  # On Windows no multiprocessing is supported, see https://github.com/predict-idlab/tsflex/issues/51
             n_jobs = 1
         elif n_jobs is None:
             n_jobs = os.cpu_count()
@@ -416,8 +652,8 @@ class FeatureCollection:
 
         if calculated_feature_list is None:
             raise RuntimeError(
-                "Feature Extraction halted due to error while extracting one (or multiple) feature(s)! "
-                + "See stack trace above."
+                "Feature Extraction halted due to error while extracting one "
+                + "(or multiple) feature(s)! See stack trace above."
             )
 
         if return_df:
@@ -474,11 +710,18 @@ class FeatureCollection:
 
         """
         # dict in which we store all the { output_col_name : (UUID, FeatureDescriptor) }
-        # items of our current Featurecollection object
+        # items of our current FeatureCollection object
+        manual_window = False
+        if any(c.endswith("w=manual") for c in feat_cols_to_keep):
+            assert all(c.endswith("w=manual") for c in feat_cols_to_keep)
+            # As the windows are created manual, the FeatureCollection cannot contain
+            # multiple windows for the same output name - input_series combination
+            self._check_no_multiple_windows()
+            manual_window = True
         feat_col_fd_mapping: Dict[str, Tuple[str, FeatureDescriptor]] = {}
-        for (s_names, window, stride), fds in self._feature_desc_dict.items():
-            fd: FeatureDescriptor
-            for fd in fds:
+        for (s_names, window), fd_list in self._feature_desc_dict.items():
+            window = "manual" if manual_window else self._ws_to_str(window)
+            for fd in fd_list:
                 # As a single FeatureDescriptor can have multiple output col names, we
                 # create a unique identifier for each FeatureDescriptor (on which we
                 # will apply set-like operations later on to only retain all the unique
@@ -486,14 +729,8 @@ class FeatureCollection:
                 uuid_str = str(uuid.uuid4())
                 for output_name in fd.function.output_names:
                     # Reconstruct the feature column name
-                    feat_col_name = "__".join(
-                        [
-                            "|".join(s_names)
-                            if isinstance(s_names, tuple)
-                            else s_names,
-                            output_name,
-                            f"w={self._ws_to_str(window)}_s={self._ws_to_str(stride)}",
-                        ]
+                    feat_col_name = StridedRolling.construct_output_index(
+                        series_keys=s_names, feat_name=output_name, win_str=window
                     )
                     feat_col_fd_mapping[feat_col_name] = (uuid_str, fd)
 
@@ -520,6 +757,7 @@ class FeatureCollection:
 
     @staticmethod
     def _ws_to_str(window_or_stride: Any) -> str:
+        """Convert the window/stride value to a (shortend) string representation."""
         if isinstance(window_or_stride, pd.Timedelta):
             return timedelta_to_str(window_or_stride)
         else:
@@ -532,13 +770,16 @@ class FeatureCollection:
         for feature_key in feature_keys:
             output_str += f"{'|'.join(feature_key)}: ("
             keys = (x for x in self._feature_desc_dict.keys() if x[0] == feature_key)
-            for _, win_size, stride in keys:
+            for _, win_size in keys:
                 output_str += f"\n\twin: "
                 win_str = self._ws_to_str(win_size)
-                stride_str = self._ws_to_str(stride)
-                output_str += f"{win_str:<6}, stride: {stride_str}: ["
-                for feat_desc in self._feature_desc_dict[feature_key, win_size, stride]:
-                    output_str += f"\n\t\t{feat_desc._func_str},"
+                output_str += f"{win_str:<6}: ["
+                for feat_desc in self._feature_desc_dict[feature_key, win_size]:
+                    stride_str = feat_desc.stride
+                    if stride_str is not None:
+                        stride_str = [self._ws_to_str(s) for s in stride_str]
+                    output_str += f"\n\t\t{feat_desc._func_str}"
+                    output_str += f"    stride: {stride_str},"
                 output_str += "\n\t]"
             output_str += "\n)\n"
         return output_str
