@@ -42,7 +42,7 @@ def seglearn_wrapper(func: Callable, func_name: Optional[str] = None) -> FuncWra
     output_names = _get_name(func) if func_name is None else func_name
     # A bit hacky (hard coded), bc hist is only func that returns multiple values
     if hasattr(func, "bins"):
-        output_names = [output_names+f"_bin{idx}" for idx in range(1, func.bins+1)]
+        output_names = [output_names + f"_bin{idx}" for idx in range(1, func.bins + 1)]
     return FuncWrapper(wrap_func, output_names=output_names)
 
 
@@ -153,14 +153,14 @@ def tsfel_feature_dict_wrapper(features_dict: Dict) -> List[Callable]:
             if nb_outputs == 1:
                 return func_name
             else:
-                return [func_name+f"_{idx}" for idx in range(1,nb_outputs+1)]
+                return [func_name + f"_{idx}" for idx in range(1, nb_outputs + 1)]
         output_param = eval(config["parameters"][nb_outputs])
-        return [func_name+f"_{nb_outputs}=v" for v in output_param]
+        return [func_name + f"_{nb_outputs}={v}" for v in output_param]
 
     functions = []
     tsfel_mod = importlib.import_module("tsfel.feature_extraction")
-    for donain_feats in features_dict.values():  # Iterate over feature domains
-        for config in donain_feats.values():  # Iterate over function configs
+    for domain_feats in features_dict.values():  # Iterate over feature domains
+        for config in domain_feats.values():  # Iterate over function configs
             func = getattr(tsfel_mod, config["function"].split(".")[-1])
             params = config["parameters"] if config["parameters"] else {}
             output_names = get_output_names(config)
@@ -208,7 +208,7 @@ def tsfresh_combiner_wrapper(func: Callable, param: List[Dict]) -> FuncWrapper:
     )
 
 
-def tsfresh_settings_wrapper(settings: Dict) -> List[Callable]:
+def tsfresh_settings_wrapper(settings: Dict) -> List[Union[Callable, FuncWrapper]]:
     """Wrapper enabling compatibility with tsfresh feature extraction settings.
 
     [tsfresh feature extraction settings](https://tsfresh.readthedocs.io/en/latest/text/feature_extraction_settings.html)
@@ -249,7 +249,7 @@ def tsfresh_settings_wrapper(settings: Dict) -> List[Callable]:
 
     Returns
     -------
-    List[Callable]
+    List[Union[Callable, FuncWrapper]]
         List of the (wrapped) tsfresh functions that are now directly compatible with
         with tsflex.
  
@@ -264,7 +264,11 @@ def tsfresh_settings_wrapper(settings: Dict) -> List[Callable]:
             functions.append(tsfresh_combiner_wrapper(func, param))
         else:
             for kwargs in param:
-                functions.append(FuncWrapper(func, output_names=f"{func.__name__}_{str(kwargs)}", **kwargs))
+                functions.append(
+                    FuncWrapper(
+                        func, output_names=f"{func.__name__}_{str(kwargs)}", **kwargs
+                    )
+                )
     return functions
 
 
@@ -274,14 +278,15 @@ def catch22_wrapper(catch22_all: Callable) -> FuncWrapper:
 
     [catch22](https://github.com/chlubba/catch22) is a collection of 22 time series 
     features that are a high-performing subset of the over 7000 features in hctsa.
+    -> [Python bindings](https://github.com/DynamicsAndNeuralSystems/pycatch22)
 
-    By using this wrapper, we can plug the catch22 features in a tsflex 
-    ``FeatureCollection``. 
+    By using this wrapper, we can plug the catch22 features in a tsflex
+    ``FeatureCollection``.
     This enables to easily extract the catch22 features while leveraging the flexibility
     of tsflex.
 
     .. Note::
-        This wrapper wraps the `catch22_all` function from `catch22`.
+        This wrapper wraps the `catch22_all` function from `pycatch22`.
         See more [here](https://github.com/chlubba/catch22/blob/master/wrap_Python/catch22/catch22.py).
 
     Example
@@ -289,7 +294,7 @@ def catch22_wrapper(catch22_all: Callable) -> FuncWrapper:
     ```python
     from tsflex.features import FeatureCollection, MultipleFeatureDescriptors
     from tsflex.features.integrations import catch22_wrapper
-    from catch22 import catch22_all
+    from pycatch22 import catch22_all
 
     catch22_feats = MultipleFeatureDescriptors(
         functions=catch22_wrapper(catch22_all),
@@ -304,7 +309,7 @@ def catch22_wrapper(catch22_all: Callable) -> FuncWrapper:
     Parameters
     ----------
     catch22_all: Callable
-        The `catch22_all` function from the `catch22` package.
+        The `catch22_all` function from the `pycatch22` package.
 
     Returns
     -------
@@ -314,92 +319,9 @@ def catch22_wrapper(catch22_all: Callable) -> FuncWrapper:
  
     """
     catch22_names = catch22_all([0])["names"]
+
     def wrap_catch22_all(x):
         return catch22_all(x)["values"]
 
     wrap_catch22_all.__name__ = "[wrapped]__" + _get_name(catch22_all)
     return FuncWrapper(wrap_catch22_all, output_names=catch22_names)
-
-
-# ----------------------------------- TRANSFORMER -----------------------------------
-from .utils import _get_funcwrapper_func_and_kwargs
-# class WrappedTransformer:
-    
-#     def __init__(self, transformer):
-#         self.transformer = transformer
-#         self.__name__ = "[wrapped_transformer]__" + transformer.__repr__()
-
-#     def __call__(self, X):
-#         # if transformer.is_fitted:
-#         try:
-#             return self.transformer.transform(X)
-#         except:
-#             print("fit_transform ")
-#             # transformer = transformer.fit(X)
-#             return self.transformer.fit_transform(X)
-
-#     def __deepcopy__(self, memo):
-#         cls = self.transformer.__class__
-#         result = cls.__new__(cls)
-#         memo[id(self.transformer)] = result
-#         from copy import deepcopy
-#         # result.__dict__.update(copy(self.transformer.__dict__))
-#         for k, v in self.transformer.__dict__.items():
-#             setattr(result, k, deepcopy(v, memo))
-#         print(result)
-#         return WrappedTransformer(result)
-
-#     # def __deepcopy__(self, memo):
-#     #     cls = self.__class__
-#     #     result = cls.__new__(cls)
-#     #     memo[id(self)] = result
-#     #     for k, v in self.__dict__.items():
-#     #         setattr(result, k, deepcopy(v, memo))
-#     #     return result
-        
-
-# def transformer_wrapper(transformer) -> FuncWrapper:
-#     from sklearn.base import TransformerMixin
-#     # assert isinstance(transformer, FuncWrapper) or isinstance(transformer, TransformerMixin)
-    
-#     func_wrapper_kwargs = {}
-#     if isinstance(transformer, FuncWrapper):
-#         # Extract the function and keyword arguments from the function wrapper
-#         transformer, func_wrapper_kwargs = _get_funcwrapper_func_and_kwargs(transformer)
-
-#     assert hasattr(transformer, "fit_transform")# and hasattr(transformer, "transform")
-
-#     func_wrapper_kwargs["vectorized"] = True
-    
-#     # from copy import copy
-
-#     wrapped_transformer = WrappedTransformer(transformer)
-#     # wrapped_transformer.__name__ = "[wrapped_transformer]__" + transformer.__repr__()
-#     return FuncWrapper(wrapped_transformer, **func_wrapper_kwargs)
-
-from .utils import _get_funcwrapper_func_and_kwargs
-def transformer_wrapper(transformer) -> FuncWrapper:
-    from sklearn.base import TransformerMixin
-    # assert isinstance(transformer, FuncWrapper) or isinstance(transformer, TransformerMixin)
-    
-    func_wrapper_kwargs = {}
-    if isinstance(transformer, FuncWrapper):
-        # Extract the function and keyword arguments from the function wrapper
-        transformer, func_wrapper_kwargs = _get_funcwrapper_func_and_kwargs(transformer)
-
-    assert hasattr(transformer, "fit") and hasattr(transformer, "transform")
-
-    func_wrapper_kwargs["vectorized"] = True
-    
-    def wrap_transformer(X):
-        try:
-            res = transformer.transform(X)
-            print("Transforming")
-            return res
-        except:
-            print("Fitting + Transforming")
-            return transformer.fit_transform(X)
-
-    wrap_transformer.__name__ = "[wrapped_transformer]__" + transformer.__repr__()
-    return FuncWrapper(wrap_transformer, **func_wrapper_kwargs)
-
