@@ -9,6 +9,7 @@ FeatureCollection: its `logging_file_path` of the `calculate` method.
 __author__ = "Jeroen Van Der Donckt"
 
 import logging
+import numpy as np
 import pandas as pd
 import re
 
@@ -31,8 +32,11 @@ def _parse_message(message: str) -> list:
     matches = re.findall(regex, remove_inner_brackets(message))
     assert len(matches) == 4
     func = matches[0]
-    key = matches[1].replace("'", "") 
-    window, stride = matches[2].split(",")[0], matches[2].split(",")[1]
+    key = matches[1].replace("'", "")
+    window = matches[2].split(",")[0].strip()
+    stride = ",".join(matches[2].split(",")[1:]).strip()
+    if stride != "manual":
+        stride = eval(stride)  # parse the tuple
     duration_s = float(matches[3].rstrip(" seconds"))
     return [func, key, window, stride, duration_s]
 
@@ -44,7 +48,7 @@ def _parse_logging_execution_to_df(logging_file_path: str) -> pd.DataFrame:
     ----------
     logging_file_path: str
         The file path where the logged messages are stored. This is the file path that
-        is passed to the `FeatureCollection` its `calculate` method. 
+        is passed to the `FeatureCollection` its `calculate` method.
 
     Note
     ----
@@ -54,7 +58,7 @@ def _parse_logging_execution_to_df(logging_file_path: str) -> pd.DataFrame:
     Returns
     -------
     pd.DataFrame
-        A DataFrame with the features its function, input series names and 
+        A DataFrame with the features its function, input series names and
         calculation duration.
 
     """
@@ -63,8 +67,32 @@ def _parse_logging_execution_to_df(logging_file_path: str) -> pd.DataFrame:
         list(df["message"].apply(_parse_message)),
         index=df.index,
     )
-    df["window"] = pd.to_timedelta(df["window"]).apply(timedelta_to_str)
-    df["stride"] = pd.to_timedelta(df["stride"]).apply(timedelta_to_str)
+    # Parse the window
+    if (df["window"] == "manual").any():
+        # All should be manual
+        assert (df["window"] == "manual").all()
+    elif df["window"].str.isnumeric().all():
+        df["window"] = pd.to_numeric(df["window"])
+    else:
+        df["window"] = pd.to_timedelta(df["window"]).apply(timedelta_to_str)
+    # Parse the stride
+    if (df["stride"] == "manual").any():
+        # All should be manual
+        assert (df["stride"] == "manual").all()
+    elif (
+        df["stride"]
+        .apply(lambda stride_tuple: np.char.isnumeric(stride_tuple).all())
+        .all()
+    ):
+        df["stride"] = df["stride"].apply(
+            lambda stride_tuple: tuple(sorted(pd.to_numeric(s) for s in stride_tuple))
+        )
+    else:
+        df["stride"] = df["stride"].apply(
+            lambda stride_tuple: tuple(
+                timedelta_to_str(pd.to_timedelta(s)) for s in stride_tuple
+            )
+        )
     return df.drop(columns=["name", "log_level", "message"])
 
 
@@ -80,7 +108,7 @@ def get_feature_logs(logging_file_path: str) -> pd.DataFrame:
     Returns
     -------
     pd.DataFrame
-        A DataFrame with the features its function, input series names and 
+        A DataFrame with the features its function, input series names and
         calculation duration.
 
     """
@@ -101,7 +129,7 @@ def get_function_stats(logging_file_path: str) -> pd.DataFrame:
     Returns
     -------
     pd.DataFrame
-        A DataFrame with for each function (i.e., `function-(window,stride)`) 
+        A DataFrame with for each function (i.e., `function-(window,stride)`)
         combination the mean (time), std (time), sum (time), and number of executions.
 
     """
@@ -118,6 +146,7 @@ def get_function_stats(logging_file_path: str) -> pd.DataFrame:
         if all(idx in sorted_funcs for idx in idx_level):
             return [sorted_funcs.index(idx) for idx in idx_level]
         return idx_level
+
     return (
         df.groupby(["function", "window", "stride"])
         .agg({"duration": ["mean", "std", "sum", "count"]})
