@@ -24,8 +24,8 @@ import pandas as pd
 
 from ..function_wrapper import FuncWrapper
 from ..logger import logger
-from ..utils import _determine_bounds
-from ...utils.data import SUPPORTED_STROLL_TYPES, to_series_list
+from ..utils import _determine_bounds, _check_start_end_array
+from ...utils.data import SUPPORTED_STROLL_TYPES, to_series_list, to_list, to_tuple
 from ...utils.attribute_parsing import DataType, AttributeParser
 from ...utils.time import timedelta_to_str
 
@@ -38,20 +38,34 @@ class StridedRolling(ABC):
 
     Parameters
     ----------
-    data : Union[pd.Series, pd.DataFrame]
+    data : Union[pd.Series, pd.DataFrame, List[Union[pd.Series, pd.DataFrame]]]
         ``pd.Series`` or ``pd.DataFrame`` to slide over, the index must be either
         numeric or a ``pd.DatetimeIndex``.
     window : Union[float, pd.Timedelta]
         Either an int, float, or ``pd.Timedelta``, representing the sliding window size
         in terms of the index (in case of a int or float) or the sliding window duration
         (in case of ``pd.Timedelta``).
-    stride : Union[float, pd.Timedelta]
-        Either an int, float, or ``pd.Timedelta``, representing the stride size in terms
-        of the index (in case of a int or float) or the stride duration (in case of
-        ``pd.Timedelta``).
+    strides : Union[float, pd.Timedelta, List[Union[float, pd.Timedelta]]], optional
+        Either a list of int, float, or ``pd.Timedelta``, representing the stride sizes
+        in terms of the index (in case of a int or float) or the stride duration (in
+        case of ``pd.Timedelta``). By default None.
+    segment_start_idxs: np.ndarray, optional
+        The start indices for the segmented windows. If not provided, the start indices
+        will be computed from the data using the passed ``strides`` or by using the
+        ``segment_end_idxs`` (if not none) + ``window``. By default None.
+    segment_end_idxs: np.ndarray, optional
+        The end indices for the segmented windows. If not provided, the end indices will
+        be computed from either (1) the data using the passed ``window`` + ``strides``
+        or (2) the ``segment_start_idxs`` + ``window``, By default None.
+        .. Note::
+            When you pass arrays to both ``segment_start_idxs`` and
+            ``segment_end_idxs``, the corresponding index-values of these arrays will be
+            used as segment-ranges. As a result, the following properties must be met:\n
+              - both arrays should have equal length
+              - all values in ``segment_start_idxs`` should be <= ``segment_end_idxs``
     start_idx: Union[float, pd.Timestamp], optional
         The start-index which will be used for each series passed to `data`. This is
-        especially useful if multiple `StridedRolling` instances are created and the
+        especially useful if multiple ``StridedRolling`` instances are created and the
         user want to ensure same (start-)indexes for each of them.
     end_idx: Union[float, pd.Timestamp], optional
         The end-index which will be used as sliding end-limit for each series passed to
@@ -62,12 +76,36 @@ class StridedRolling(ABC):
         .. Note::
             Make sure to only set this argument to pd.Series when this is really
             required, since pd.Series strided-rolling is significantly less efficient.
-            For a np.array it is possible to create very efficient views, but there is no
-            such thing as a pd.Series view. Thus, for each stroll, a new series is created.
+            For a np.array it is possible to create very efficient views, but there is
+            no such thing as a pd.Series view. Thus, for each stroll, a new series is
+            created, inducing a lot of non-feature calculation of overhead.
     window_idx : str, optional
         The window's index position which will be used as index for the
         feature_window aggregation. Must be either of: `["begin", "middle", "end"]`, by
         default "end".
+    include_final_window: bool, optional
+        Whether the final (possibly incomplete) window should be included in the
+        strided-window segmentation, by default False.
+
+        .. Note::
+            The remarks below apply when ``include_final_window`` is set to True.
+            The user should be aware that the last window *might* be incomplete, i.e.;
+
+            - when equally sampled, the last window *might* be smaller than the
+              the other windows.
+            - when not equally sampled, the last window *might* not include all the
+                data points (as the begin-time + window-size comes after the last data
+                point).
+
+            Note, that when equally sampled, the last window *will* be a full window
+            when:
+
+            - the stride is the sampling rate of the data (or stride = 1 for
+              sample-based configurations).<br>
+              **Remark**: that when `include_final_window` is set to False, the last
+              window (which is a full) window will not be included!
+            - *(len * sampling_rate - window_size) % stride = 0*. Remark that the above
+              case is a base case of this.
     approve_sparsity: bool, optional
         Bool indicating whether the user acknowledges that there may be sparsity (i.e.,
         irregularly sampled data), by default False.
@@ -79,9 +117,12 @@ class StridedRolling(ABC):
 
     """
 
-    # Class variables which are used by sub-classes
+    # Class variables which are used by subclasses
     win_str_type: DataType
     reset_series_index_b4_segmenting: bool = False
+    OUTSIDE_DATA_BOUNDS_WARNING: str = (
+        "Some segment indexes are outside the range of the data its index."
+    )
 
     # Create the named tuple
     _NumpySeriesContainer = namedtuple(
@@ -92,33 +133,48 @@ class StridedRolling(ABC):
         self,
         data: Union[pd.Series, pd.DataFrame, List[Union[pd.Series, pd.DataFrame]]],
         window: T,
-        stride: T,
+        strides: Optional[Union[T, List[T]]] = None,
+        segment_start_idxs: Optional[np.ndarray] = None,
+        segment_end_idxs: Optional[np.ndarray] = None,
         start_idx: Optional[T] = None,
         end_idx: Optional[T] = None,
         func_data_type: Optional[Union[np.array, pd.Series]] = np.array,
         window_idx: Optional[str] = "end",
+        include_final_window: bool = False,
         approve_sparsity: Optional[bool] = False,
     ):
-        self.window = window
-        self.stride = stride
+        if strides is not None:
+            strides = to_list(strides)
 
-        # Note: the sub-class should set the self.func_data_type attribute
-        assert AttributeParser.check_expected_type([window, stride], self.win_str_type)
+        # Check the passed segment indices
+        if segment_start_idxs is not None and segment_end_idxs is not None:
+            _check_start_end_array(segment_start_idxs, segment_end_idxs)
+
+        if window is not None:
+            assert AttributeParser.check_expected_type(
+                [window] + ([] if strides is None else strides), self.win_str_type
+            )
+
+        self.window = window
+        self.strides = strides
 
         self.window_idx = window_idx
+        self.include_final_window = include_final_window
         self.approve_sparsity = approve_sparsity
 
         assert func_data_type in SUPPORTED_STROLL_TYPES
         self.data_type = func_data_type
 
         # 0. Standardize the input
-        series_list: List[pd.Series] = to_series_list(data)  # TODO: isn't it always a list of series?
+        series_list: List[pd.Series] = to_series_list(data)
         self.series_dtype = AttributeParser.determine_type(series_list)
         self.series_key: Tuple[str, ...] = tuple([str(s.name) for s in series_list])
 
         # 1. Determine the start index
         self.start, self.end = start_idx, end_idx
         if self.start is None or self.end is None:
+            # We always pass start_idx and end_idx from the FeatureCollection.calculate
+            # Hence, this code is only useful for testing purposes
             start, end = _determine_bounds("inner", series_list)
 
             # update self.start & self.end if it was not passed
@@ -128,43 +184,103 @@ class StridedRolling(ABC):
         # Especially useful when the index dtype differs from the win-stride-dtype
         # e.g. -> performing a int-based stroll on time-indexed data
         # Note: this is very niche and thus requires advanced knowledge
+        # TODO: this code can be omitted if we remove TimeIndexSampleStridedRolling
         self._update_start_end_indices_to_stroll_type(series_list)
 
-        # 2. Create a new-index which will be used for DataFrame reconstruction
-        # Note: the index-name of the first passed series will be re-used as index-name
-        self.index = self._construct_output_index(series_list[0])
+        # 2. Construct the index ranges
+        # Either use the passed segment indices or compute the start or end times of the
+        # segments. The segment indices have precedence over the stride (and window) for
+        # index computation.
+        if segment_start_idxs is not None or segment_end_idxs is not None:
+            self.strides = None
+            if segment_start_idxs is not None and segment_end_idxs is not None:
+                # When both the start and end points are passed, the window does not
+                # matter.
+                self.window = None
+                np_start_times = self._parse_segment_idxs(segment_start_idxs)
+                np_end_times = self._parse_segment_idxs(segment_end_idxs)
+            elif segment_start_idxs is not None:  # segment_end_idxs is None
+                np_start_times = self._parse_segment_idxs(segment_start_idxs)
+                np_end_times = np_start_times + self._get_np_value(self.window)
+            else:  # segment_end_idxs is not None and segment_start_idxs is None
+                np_end_times = self._parse_segment_idxs(segment_end_idxs)
+                np_start_times = np_end_times - self._get_np_value(self.window)
+        else:
+            np_start_times = self._construct_start_idxs()
+            np_end_times = np_start_times + self._get_np_value(self.window)
 
-        # 3. Construct the index ranges and store the series containers
-        np_start_times, np_end_times = self._construct_start_end_times()
+        # Check the numpy start and end indices
+        _check_start_end_array(np_start_times, np_end_times)
+
+        # 3. Create a new-index which will be used for DataFrame reconstruction
+        # Note: the index-name of the first passed series will be re-used as index-name
+        self.index = self._get_output_index(
+            np_start_times, np_end_times, name=series_list[0].index.name
+        )
+
+        # 4. Store the series containers
         self.series_containers = self._construct_series_containers(
             series_list, np_start_times, np_end_times
         )
 
-        # 4. Check the sparsity assumption
-        if not self.approve_sparsity:
-            qs = [0, 0.1, 0.5, 0.9, 1]
+        # 5. Check the sparsity assumption
+        if not self.approve_sparsity and len(self.index):
             for container in self.series_containers:
-                series_idx_stats = np.quantile(
-                    container.end_indexes - container.start_indexes, q=qs
-                )
-                q_str = ", ".join([f"q={q}: {v}" for q, v in zip(qs, series_idx_stats)])
                 # Warn when min != max
-                if not all(series_idx_stats == series_idx_stats[-1]):
+                if np.ptp(container.end_indexes - container.start_indexes) != 0:
                     warnings.warn(
                         f"There are gaps in the sequence of the {container.name}"
-                        f"-series;\n \t Quantiles of nb values in window: {q_str}",
+                        f"-series!",
                         RuntimeWarning,
                     )
 
-    def _get_window_offset(self, window: T) -> T:
+    def _calc_nb_segments_for_stride(self, stride) -> int:
+        """Calculate the number of output items (segments) for a given single stride."""
+        nb_feats = max((self.end - self.start - self.window) // stride + 1, 0)
+        # Add 1 if there is still some data after (including) the last window its
+        # start index - this is only added when `include_last_window` is True.
+        nb_feats += self.include_final_window * (
+            self.start + stride * nb_feats <= self.end
+        )
+        return nb_feats
+
+    def _get_np_start_idx_for_stride(self, stride: T) -> np.ndarray:
+        """Compute the start index for the given single stride."""
+        # ---------- Efficient numpy code -------
+        np_start = self._get_np_value(self.start)
+        np_stride = self._get_np_value(stride)
+        # Compute the start times (these remain the same for each series)
+        return np.arange(
+            start=np_start,
+            stop=np_start + self._calc_nb_segments_for_stride(stride) * np_stride,
+            step=np_stride,
+        )
+
+    def _construct_start_idxs(self) -> np.ndarray:
+        """Construct the start indices of the segments (for all stride values).
+
+        To realize this, we compute the start idxs for each stride and then merge them
+        together (without duplicates) in a sorted array.
+        """
+        start_idxs = []
+        for stride in self.strides:
+            start_idxs += [self._get_np_start_idx_for_stride(stride)]
+        # note - np.unique also sorts the array
+        return np.unique(np.concatenate(start_idxs))
+
+    def _get_output_index(
+        self, start_idxs: np.ndarray, end_idxs: Union[np.ndarray, None], name: str
+    ) -> pd.Index:
+        """Construct the output index."""
         if self.window_idx == "end":
-            return window
+            return pd.Index(end_idxs, name=name)
         elif self.window_idx == "middle":
-            return window / 2
+            return pd.Index(
+                start_idxs + ((end_idxs - start_idxs) / 2),
+                name=name,
+            )
         elif self.window_idx == "begin":
-            if isinstance(window, pd.Timedelta):
-                return pd.Timedelta(seconds=0)
-            return 0
+            return pd.Index(start_idxs, name=name)
         else:
             raise ValueError(
                 f"window index {self.window_idx} must be either of: "
@@ -181,6 +297,7 @@ class StridedRolling(ABC):
                 np_idx_times = series.index.values
             else:
                 np_idx_times = np.arange(len(series))
+                # note: using pd.RangeIndex instead of arange gives the same performance
 
             series_name = series.name
             if self.data_type is np.array:
@@ -221,7 +338,7 @@ class StridedRolling(ABC):
         pd.DataFrame
             The merged output of the function applied to every column in a
             new DataFrame. The DataFrame's column-names have the format:
-                `<series_col_name(s)>_<feature_name>__w=<window>_s=<stride>`.
+                `<series_col_name(s)>_<feature_name>__w=<window>`.
 
         Raises
         ------
@@ -250,7 +367,7 @@ class StridedRolling(ABC):
         # expression only once, whereas a list comprehension evaluates its expression
         # every time).
         # See more why: https://stackoverflow.com/a/59838723
-        out: np.array = None
+        out: np.array
         if func.vectorized:
             # Vectorized function execution
 
@@ -271,7 +388,7 @@ class StridedRolling(ABC):
 
             ## IMPL 2
             ## Is a good implementation (equivalent to the one below), will also fail in
-            ## the same cases, but it does not perform clear assertions (with their 
+            ## the same cases, but it does not perform clear assertions (with their
             ## accompanied clear messages).
             # out = np.asarray(
             #     func(
@@ -284,18 +401,42 @@ class StridedRolling(ABC):
 
             views = []
             for sc in self.series_containers:
-                windows = sc.end_indexes - sc.start_indexes
-                strides = sc.start_indexes[1:] - sc.start_indexes[:-1]
-                assert np.all(
-                    windows == windows[0]
-                ), "Vectorized functions require same number of samples in each segmented window!"
-                assert np.all(
-                    strides == strides[0]
-                ), "Vectorized functions require same number of samples as stride!"
-                views.append(
-                    _sliding_strided_window_1d(sc.values, windows[0], strides[0])
-                )
-            out = func(*views)
+                if len(sc.start_indexes) == 0:
+                    # There are no feature windows  -> return empty array (see below)
+                    views = []
+                    break
+                elif len(sc.start_indexes) == 1:
+                    # There is only 1 feature window (bc no steps in the sliding window)
+                    views.append(
+                        np.expand_dims(
+                            sc.values[sc.start_indexes[0] : sc.end_indexes[0]],
+                            axis=0,
+                        )
+                    )
+                else:
+                    # There are >1 feature windows (bc >=1 steps in the sliding window)
+                    windows = sc.end_indexes - sc.start_indexes
+                    strides = sc.start_indexes[1:] - sc.start_indexes[:-1]
+                    assert np.all(windows == windows[0]), (
+                        "Vectorized functions require same number of samples in each "
+                        + "segmented window!"
+                    )
+                    assert np.all(
+                        strides == strides[0]
+                    ), "Vectorized functions require same number of samples as stride!"
+                    views.append(
+                        _sliding_strided_window_1d(
+                            sc.values[sc.start_indexes[0] :],
+                            windows[0],
+                            strides[0],
+                            len(self.index),
+                        )
+                    )
+
+            # Assign empty array as output when there is no view to apply the vectorized
+            # function on (this is the case when there is at least for one series no
+            # feature windows)
+            out = func(*views) if len(views) >= 1 else np.array([])
 
             out_type = type(out)
             out = np.asarray(out)
@@ -327,10 +468,17 @@ class StridedRolling(ABC):
 
         # Aggregate function output in a dictionary
         feat_out = {}
-        if out.ndim == 1 or (out.ndim == 2 and out.shape[1] == 1):
+        if out.ndim == 1 and not len(out):
+            # When there are no features calculated (due to no feature windows)
+            assert not len(self.index)
+            for f_name in feat_names:
+                # Will be discarded (bc no index)
+                feat_out[self._create_feat_col_name(f_name)] = None
+        elif out.ndim == 1 or (out.ndim == 2 and out.shape[1] == 1):
             assert len(feat_names) == 1, f"Func {func} returned more than 1 output!"
             feat_out[self._create_feat_col_name(feat_names[0])] = out.flatten()
-        if out.ndim == 2 and out.shape[1] > 1:
+        else:
+            assert out.ndim == 2 and out.shape[1] > 1
             assert (
                 len(feat_names) == out.shape[1]
             ), f"Func {func} returned incorrect number of outputs ({out.shape[1]})!"
@@ -340,24 +488,47 @@ class StridedRolling(ABC):
                 ]
 
         elapsed = time.time() - t_start
+        log_strides = (
+            "manual" if self.strides is None else tuple(map(str, self.strides))
+        )
+        log_window = "manual" if self.window is None else self.window
         logger.info(
             f"Finished function [{func.func.__name__}] on "
             f"{[self.series_key]} with window-stride "
-            f"[{self.window}, {self.stride}] in [{elapsed} seconds]!"
+            f"[{log_window}, {log_strides}] in [{elapsed} seconds]!"
         )
 
         return pd.DataFrame(index=self.index, data=feat_out)
 
-    def _update_start_end_indices_to_stroll_type(self, series_list: List[pd.Series]):
-        pass
+    # --------------------------------- STATIC METHODS ---------------------------------
+    @staticmethod
+    def _get_np_value(val):
+        # Convert everything to int64
+        if isinstance(val, pd.Timestamp):
+            return val.to_datetime64()
+        elif isinstance(val, pd.Timedelta):
+            return val.to_timedelta64()
+        else:
+            return val
+
+    @staticmethod
+    def construct_output_index(
+        series_keys: Union[str, Tuple[str, ...]], feat_name: str, win_str: str
+    ) -> str:
+        series_keys = to_tuple(series_keys)
+        return f"{'|'.join(series_keys)}__{feat_name}__w={win_str}"
 
     # ----------------------------- OVERRIDE THESE METHODS -----------------------------
     @abstractmethod
-    def _construct_output_index(self, series: pd.Series) -> pd.Index:
+    def _update_start_end_indices_to_stroll_type(self, series_list: List[pd.Series]):
+        # NOTE: This method will only be implemented (with code != pass) in the 
+        # TimeIndexSampleStridedRolling
         raise NotImplementedError
 
     @abstractmethod
-    def _construct_start_end_times(self) -> Tuple[np.ndarray, np.ndarray]:
+    def _parse_segment_idxs(self, segment_idxs: np.ndarray) -> np.ndarray:
+        """Check the segment indexes array to lie between self.start and self.end and
+        convert it to the correct dtype (if necessary)."""
         raise NotImplementedError
 
     @abstractmethod
@@ -370,56 +541,31 @@ class SequenceStridedRolling(StridedRolling):
         self,
         data: Union[pd.Series, pd.DataFrame, List[Union[pd.Series, pd.DataFrame]]],
         window: float,
-        stride: float,
+        strides: Optional[Union[float, List[float]]] = None,
         *args,
         **kwargs,
     ):
+        # Set the data type & call the super constructor
         self.win_str_type = DataType.SEQUENCE
-        super().__init__(data, window, stride, *args, **kwargs)
+        super().__init__(data, window, strides, *args, **kwargs)
 
-    # ------------------------------ Overridden methods ------------------------------
-    def _construct_output_index(self, series: pd.Series) -> pd.Index:
-        window_offset = self._get_window_offset(self.window)
-        # bool which indicates whether the `end` lies on the boundary
-        # and as arange does not include the right boundary -> use it to enlarge `stop`
-        boundary = (self.end + 1 - self.start - self.window) % self.stride <= 1
-        # print((self.end + 1 - self.start - self.window) % self.stride )
-        # print("last idx:", self.end, "nb samples:", len(series))
-        # idx = np.arange(
-        #         start=self.start + window_offset,
-        #         stop=self.end - self.window + window_offset + self.stride * boundary,
-        #         step=self.stride,
-        #     )
-        # print("END IDX:", idx)
-        return pd.Index(
-            data=np.arange(
-                start=self.start + window_offset,
-                stop=self.end - self.window + window_offset + self.stride * boundary,
-                step=self.stride,
-            ),
-            name=series.index.name,
-        )
+    # ------------------------------- Overridden methods -------------------------------
+    def _update_start_end_indices_to_stroll_type(self, series_list: List[pd.Series]):
+        pass
 
-    def _construct_start_end_times(self) -> Tuple[np.ndarray, np.ndarray]:
-        # ---------- Efficient numpy code -------
-        # 1. Precompute the start & end times (these remain the same for each series)
-        # note: this if equivalent to:
-        #   if `window` == 'begin":
-        #       start_times = self.index.values
-        np_start_times = np.arange(
-            start=self.start,
-            stop=self.start + (len(self.index) * self.stride),
-            step=self.stride,
-            dtype=self.index.dtype,
-        )
-        np_end_times = np_start_times + self.window
-        return np_start_times, np_end_times
+    def _parse_segment_idxs(self, segment_idxs: np.ndarray) -> np.ndarray:
+        if any((segment_idxs < self.start) | (segment_idxs > self.end)):
+            warnings.warn(self.OUTSIDE_DATA_BOUNDS_WARNING, RuntimeWarning)
+        return segment_idxs
 
     def _create_feat_col_name(self, feat_name: str) -> str:
-        # TODO -> this is not that loosely coupled if we want somewhere else in the code
-        # to also reproduce col-name construction
-        win_stride_str = f"w={self.window}_s={self.stride}"
-        return f"{'|'.join(self.series_key)}__{feat_name}__{win_stride_str}"
+        if self.window is not None:
+            win_str = str(self.window)
+        else:
+            win_str = "manual"
+        return self.construct_output_index(
+            series_keys=self.series_key, feat_name=feat_name, win_str=win_str
+        )
 
 
 class TimeStridedRolling(StridedRolling):
@@ -427,59 +573,68 @@ class TimeStridedRolling(StridedRolling):
         self,
         data: Union[pd.Series, pd.DataFrame, List[Union[pd.Series, pd.DataFrame]]],
         window: pd.Timedelta,
-        stride: pd.Timedelta,
+        strides: Optional[Union[pd.Timedelta, List[pd.Timedelta]]] = None,
         *args,
         **kwargs,
     ):
+        # Check that each series / dataframe has the same tz
+        data = to_series_list(data)
+        tz_index = data[0].index.tz
+        for data_entry in to_series_list(data)[1:]:
+            assert (
+                data_entry.index.tz == tz_index
+            ), "strided rolling input data must all have same timezone"
+        self._tz_index = tz_index
+        # Set the data type & call the super constructor
         self.win_str_type = DataType.TIME
-        super().__init__(data, window, stride, *args, **kwargs)
+        super().__init__(data, window, strides, *args, **kwargs)
 
-    # ---------------------------- Overridden methods ------------------------------
-    def _construct_output_index(self, series: pd.Series) -> pd.DatetimeIndex:
-        # note: the index automatically takes the timezone of `start` & `end`
-        window_offset = self._get_window_offset(self.window)
-        return pd.date_range(
-            start=self.start + window_offset,
-            end=self.end - self.window + window_offset,
-            freq=self.stride,
-            name=series.index.name,
-            closed=None,
-        )
+    # -------------------------------- Extended methods --------------------------------
+    def _get_output_index(
+        self, start_idxs: np.ndarray, end_idxs: np.ndarray, name: str
+    ) -> pd.Index:
+        assert start_idxs.dtype.type == np.datetime64
+        assert end_idxs.dtype.type == np.datetime64
+        start_idxs = pd.to_datetime(start_idxs, utc=True).tz_convert(self._tz_index)
+        end_idxs = pd.to_datetime(end_idxs, utc=True).tz_convert(self._tz_index)
+        return super()._get_output_index(start_idxs, end_idxs, name)
 
-    def _construct_start_end_times(self) -> Tuple[np.ndarray, np.ndarray]:
-        # ---------- Efficient numpy code -------
-        # 1. Convert everything to int64
-        np_start = self.start.to_datetime64()
-        np_window = self.window.to_timedelta64()
-        np_stride = self.stride.to_timedelta64()
+    # ------------------------------- Overridden methods -------------------------------
+    def _update_start_end_indices_to_stroll_type(self, series_list: List[pd.Series]):
+        pass
 
-        # 2. Precompute the start & end times (these remain the same for each series)
-        # note: this if equivalent to:
-        #   if `window` == 'begin":
-        #       start_times = self.index.values
-        np_start_times = np.arange(
-            start=np_start,
-            stop=np_start + (len(self.index) * np_stride),
-            step=np_stride,
-            dtype=np.datetime64,
-        )
-        np_end_times = np_start_times + np_window
-        return np_start_times, np_end_times
+    def _parse_segment_idxs(self, segment_idxs: np.ndarray) -> np.ndarray:
+        segment_idxs = segment_idxs.astype("datetime64")
+        start_, end_ = self.start, self.end
+        if start_.tz is not None:
+            # Convert to UTC (allowing comparison with the segment_idxs)
+            assert end_.tz is not None
+            start_ = start_.tz_convert(None)
+            end_ = end_.tz_convert(None)
+        if any((segment_idxs < start_) | (segment_idxs > end_)):
+            warnings.warn(self.OUTSIDE_DATA_BOUNDS_WARNING, RuntimeWarning)
+        return segment_idxs
 
     def _create_feat_col_name(self, feat_name: str) -> str:
-        # Convert win & stride to time-string if available :)
-        win_str = timedelta_to_str(self.window)
-        stride_str = timedelta_to_str(self.stride)
-        win_stride_str = f"w={win_str}_s={stride_str}"
-        return f"{'|'.join(self.series_key)}__{feat_name}__{win_stride_str}"
+        # Convert win to time-string if available :)
+        if self.window is not None:
+            win_str = timedelta_to_str(self.window)
+        else:
+            win_str = "manual"
+        return self.construct_output_index(
+            series_keys=self.series_key, feat_name=feat_name, win_str=win_str
+        )
 
 
 class TimeIndexSampleStridedRolling(SequenceStridedRolling):
     def __init__(
         self,
+        # TODO -> update arguments
         data: Union[pd.Series, pd.DataFrame, List[Union[pd.Series, pd.DataFrame]]],
         window: int,
-        stride: int,
+        strides: Optional[Union[int, List[int]]] = None,
+        segment_start_idxs: Optional[np.ndarray] = None,
+        segment_end_idxs: Optional[np.ndarray] = None,
         *args,
         **kwargs,
     ):
@@ -491,8 +646,19 @@ class TimeIndexSampleStridedRolling(SequenceStridedRolling):
             - must _roughly_ **share** the same **sample frequency**.
             - will be first time-aligned before transitioning to sample-segmentation by
               using the inner bounds
+        
+        .. Note::
+            `TimeIndexSampleStridedRolling` **does not support** the 
+            ``segment_start_idxs`` and ``segment_end_idxs`` arguments. Setting these 
+            will raise a NotImplementedError.
 
         """
+        if segment_start_idxs is not None or segment_end_idxs is not None:
+            raise NotImplementedError(
+                "TimeIndexSampleStridedRolling is not implemented to support passing"
+                + "segment_start_idxs or segment_end_idxs"
+            )
+
         # We want to reset the index as its type differs from the passed win-stride
         # configs
         self.reset_series_index_b4_segmenting = True
@@ -505,13 +671,22 @@ class TimeIndexSampleStridedRolling(SequenceStridedRolling):
             series_list = [s[start:end] for s in series_list]
             kwargs.update({"start_idx": start, "end_idx": end})
 
+        # We retain the first series list to stitch back the output index
+        self._series_index = series_list[0].index
+
         # pass the sliced series list instead of data
-        super().__init__(series_list, window, stride, *args, **kwargs)
+        super().__init__(series_list, window, strides, *args, **kwargs)
 
         assert self.series_dtype == DataType.TIME
 
         # we want to assure that the window-stride arguments are integers (samples)
-        assert all(isinstance(p, int) for p in [self.window, self.stride])
+        assert all(isinstance(p, int) for p in [self.window] + self.strides)
+
+    def apply_func(self, func: FuncWrapper) -> pd.DataFrame:
+        # Apply the function and stitch back the time-index
+        df = super().apply_func(func)
+        df.index = self._series_index[df.index]
+        return df
 
     # ---------------------------- Overridden methods ------------------------------
     def _update_start_end_indices_to_stroll_type(self, series_list: List[pd.Series]):
@@ -522,43 +697,10 @@ class TimeIndexSampleStridedRolling(SequenceStridedRolling):
             "left",
         )
 
-    def _construct_output_index(self, series: pd.Series) -> pd.DatetimeIndex:
-        window_offset = int(self._get_window_offset(self.window))
-        assert all(isinstance(p, np.int64) for p in [self.start, self.end])
 
-        # Note: so we have one or multiple time-indexed series on which we specified a
-        # sample based window-stride configuration -> assumptions we make
-        # * if we have  multiple series as input for a feature-functions
-        #  -> we assume the time-indexes are (roughly) the same for each series
-
-        # bool which indicates whether the `end` lies on the boundary
-        # and as arange does not include the right boundary -> use it to enlarge `stop`
-        boundary = (self.end - self.start - self.window) % self.stride == 0
-        return series.iloc[
-            np.arange(
-                start=int(window_offset),
-                stop=self.end - self.window + window_offset + self.stride * boundary,
-                step=self.stride,
-            )
-        ].index
-
-    def _construct_start_end_times(self) -> Tuple[np.ndarray, np.ndarray]:
-        # ---------- Efficient numpy code -------
-        # 1. Precompute the start & end times (these remain the same for each series)
-        # note: this if equivalent to:
-        #   if `window` == 'begin":
-        #       start_times = self.index.values
-        np_start_times = np.arange(
-            start=self.start,
-            stop=self.start + (len(self.index) * self.stride),
-            step=self.stride,
-            dtype=np.int64,  # the only thing that is changed w.r.t. the Superclass
-        )
-        np_end_times = np_start_times + self.window
-        return np_start_times, np_end_times
-
-
-def _sliding_strided_window_1d(data: np.ndarray, window: int, step: int):
+def _sliding_strided_window_1d(
+    data: np.ndarray, window: int, step: int, nb_segments: int
+):
     """View based sliding strided-window for 1-dimensional data.
 
     Parameters
@@ -569,6 +711,9 @@ def _sliding_strided_window_1d(data: np.ndarray, window: int, step: int):
         The window size, in number of samples.
     step: int
         The step size (i.e., the stride), in number of samples.
+    nb_segments: int
+        The number of sliding window steps, this is equal to the number of feature
+        windows.
 
     Returns
     -------
@@ -578,18 +723,13 @@ def _sliding_strided_window_1d(data: np.ndarray, window: int, step: int):
     """
     # window and step in samples
     assert data.ndim == 1, "data must be 1 dimensional"
-
-    if isinstance(window, float):
-        assert window.is_integer(), "window must be an int!"
-        window = int(window)
-    if isinstance(step, float):
-        assert step.is_integer(), "step must be an int!"
-        step = int(step)
+    assert isinstance(window, (int, np.integer)), "window must be an integer"
+    assert isinstance(step, (int, np.integer)), "step must be an integer"
 
     assert (step >= 1) & (window < len(data))
 
     shape = [
-        np.floor(len(data) / step - window / step + 1).astype(int),
+        nb_segments,
         window,
     ]
 
@@ -599,5 +739,5 @@ def _sliding_strided_window_1d(data: np.ndarray, window: int, step: int):
     ]
 
     return np.lib.stride_tricks.as_strided(
-        data, shape=shape, strides=strides#, writeable=False
+        data, shape=shape, strides=strides  # , writeable=False
     )
